//
//  AnnualReportPDFPage.swift
//  sKakeibo
//
//  Created by 梅山 新一郎 on 2018/10/29.
//  Copyright © 2018年 Umeya. All rights reserved.
//

import Cocoa
import Quartz

class AnnualReportPDFPage: PDFPage {
    private let pageHeight:CGFloat
    private let pageWidth:CGFloat
    private let titlePos:NSPoint
    
    private let tableLeftPos0:NSPoint
    private let nameWidth:Int
    private let amountWidth:Int
    private let totalWidth:Int
    private let rowHeight:Int
    //private let verticalPadding:CGFloat
    //private let textInSet:CGFloat
    
    private let numberOfRows:Int
    
    private var nendo:String
    private var names:[String]
    private var data:[[Int]]
    private var rowOffsetOfIncomeTotal:Int
    private var rowOffsetOfSpenddingTotal:Int
    
    init(nendo:String, names:[String], data: [[Int]], rowOffsetOfIncomeTotal:Int, rowOffsetOfSpenddingTotal:Int){
        self.nendo = nendo
        self.names = names
        self.data = data
        // A4  25(1+24) rows per a page   :adjust for A4 landscape size:width 842 , height 595(72dpi)
        self.pageHeight = CGFloat(595)
        self.pageWidth = CGFloat(842)
        self.titlePos = NSMakePoint(50, 50)
        self.tableLeftPos0 = NSMakePoint(20, 70)
        self.nameWidth = 160
        self.amountWidth = 50
        self.totalWidth = 55
        
        self.rowHeight = 20
        //self.verticalPadding = CGFloat (2.0)
        //self.textInSet = CGFloat(4.0)
        self.numberOfRows = 24 + 1
        
        self.rowOffsetOfIncomeTotal = rowOffsetOfIncomeTotal
        self.rowOffsetOfSpenddingTotal = rowOffsetOfSpenddingTotal
        super.init()
    }
    
    override func bounds(for box: PDFDisplayBox) -> NSRect {
        return NSRect(x: 0, y: 0, width: pageWidth, height: pageHeight)
    }
    
    override func draw(with box: PDFDisplayBox) {
        super.draw(with: box)
        self.drawTableLines()
        self.drawTableData()
    }

    func rcPoint(row:Int, col:Int, wPadding:Int = 0, hPadding:Int = 0)->NSPoint{
        var x = tableLeftPos0.x
        switch col{
        case 0:
            x = tableLeftPos0.x
        case 1...13:
            x = tableLeftPos0.x + CGFloat(nameWidth + amountWidth * (col - 1))
        case 14:
            x = tableLeftPos0.x + CGFloat(nameWidth + amountWidth * 12 + totalWidth)
        default:
            print("error in rcPoint", col)
        }
        let y = tableLeftPos0.y + CGFloat(rowHeight * row)
        return NSMakePoint(x + CGFloat(wPadding), y + CGFloat(hPadding))
    }

    func drawTableLines(){
        
        drawLine(linewidth: CGFloat(1), from: rcPoint(row: 0, col: 0), to: rcPoint(row: 0, col: 14))
        drawLine(linewidth: CGFloat(1), from: rcPoint(row: 1, col: 0), to: rcPoint(row: 1, col: 14))
        drawLine(linewidth: CGFloat(1), from: rcPoint(row: 25, col: 0), to: rcPoint(row: 25, col: 14))
        drawLine(linewidth: CGFloat(1), from: rcPoint(row: 0, col: 0), to: rcPoint(row: 25, col: 0))
        drawLine(linewidth: CGFloat(1), from: rcPoint(row: 0, col: 14), to: rcPoint(row: 25, col: 14))
        drawLine(linewidth: CGFloat(0.5), from: rcPoint(row: 0, col: 1), to: rcPoint(row: 25, col: 1))
        if rowOffsetOfIncomeTotal > -1{
            drawLine(linewidth: CGFloat(0.5), from: rcPoint(row: rowOffsetOfIncomeTotal, col: 0), to: rcPoint(row: rowOffsetOfIncomeTotal, col: 14))
        }
        if rowOffsetOfSpenddingTotal > 1{
            drawLine(linewidth: CGFloat(0.5), from: rcPoint(row: rowOffsetOfSpenddingTotal, col: 0), to: rcPoint(row: rowOffsetOfSpenddingTotal, col: 14))

        }
        
    }

    func drawLine(linewidth: CGFloat, from:NSPoint, to:NSPoint){
        let path = NSBezierPath()
        NSColor.darkGray.set()
        //(0,0)left-bottom -> left-top
        let fromC = NSMakePoint(from.x, self.pageHeight - from.y)
        let toC = NSMakePoint(to.x, self.pageHeight - to.y)
        path.move(to: toC)
        path.line(to: fromC)
        path.lineWidth = linewidth
        path.stroke()
    }
    
    func drawTableData(){
        
        let DF = DateFormatter()
        DF.dateFormat = "YYYY年MM月dd日"
        DF.timeZone = NSTimeZone.local
        let strDate = DF.string(from: Date())
        let s = nendo + " 年度　レポート　" + strDate
        let titleFont = NSFont(name:"Helvetica", size: 12.0)
        var rect0 = makeRect(x: self.titlePos.x, y: self.titlePos.y, w: 300, h: 30)
        s.draw(in:rect0, withAttributes:[NSAttributedString.Key.font: titleFont!])

        let wPadding = 5; let hPadding = 2
        let itemFont = NSFont(name:"Helvetica", size: 10.0)
        var p0 = rcPoint(row: 1, col: 0, wPadding: wPadding, hPadding: hPadding)
        rect0 = makeRect(x: p0.x, y: p0.y, w: nameWidth, h: rowHeight)
        " 名　称".draw(in:rect0, withAttributes:[NSAttributedString.Key.font: itemFont!])
        for m in 1...12{
            p0 = rcPoint(row: 1, col: m, wPadding: wPadding, hPadding: hPadding)
            rect0 = makeRect(x: p0.x, y: p0.y, w: amountWidth, h: rowHeight)
            String(format: "%6d 月", m).draw(in:rect0, withAttributes:[NSAttributedString.Key.font: itemFont!])
        }
        p0 = rcPoint(row: 1, col: 13, wPadding: wPadding, hPadding: hPadding)
        rect0 = makeRect(x: p0.x, y: p0.y, w: totalWidth, h: rowHeight)
        "      計".draw(in:rect0, withAttributes:[NSAttributedString.Key.font: itemFont!])
        var r = 2
        for (name, dt ) in zip(names, data){
            p0 = rcPoint(row: r, col: 0, wPadding: wPadding, hPadding: hPadding)
            rect0 = makeRect(x: p0.x, y: p0.y, w: nameWidth, h: rowHeight)
            name.draw(in:rect0, withAttributes:[NSAttributedString.Key.font: itemFont!])
            for m in 1...12{
                p0 = rcPoint(row: r, col: m, wPadding: wPadding, hPadding: hPadding)
                rect0 = makeRect(x: p0.x, y: p0.y, w: amountWidth, h: rowHeight)
                int2stringWithComma(integer: dt[m - 1]).draw(in:rect0, withAttributes:[NSAttributedString.Key.font: itemFont!])
            }
            p0 = rcPoint(row: r, col: 13, wPadding: wPadding, hPadding: hPadding)
            rect0 = makeRect(x: p0.x, y: p0.y, w: totalWidth, h: rowHeight)
            int2stringWithComma(integer: dt[12]).draw(in:rect0, withAttributes:[NSAttributedString.Key.font: itemFont!])
            r += 1
        }
    }

    func makeRect(x:CGFloat, y:CGFloat, w:Int, h:Int)->NSRect{
        let y0 = self.pageHeight - y
        return NSMakeRect(x, y0, CGFloat(w), CGFloat(h))
    }
    func int2stringWithComma(integer:Int, digits:Int  = 10) -> String{
        var s = ""
        for _ in 1...digits{ s += " " }
        let formatter = NumberFormatter()
        formatter.numberStyle = .decimal
        s += formatter.string(from: integer as NSNumber) ?? "error"
        return String(s.suffix(digits))
    }
}
