//
//  SisanReportPDFPage.swift
//  sKakeibo
//
//  Created by 梅山 新一郎 on 2018/10/27.
//  Copyright © 2018年 Umeya. All rights reserved.
//

import Cocoa
import Quartz




class SisanReportPDFPage: PDFPage {
    
    
    private let pageHeight:CGFloat
    private let pageWidth:CGFloat
    private let titlePos:NSPoint
    private let datePos:NSPoint
    private let tableLeftPos0:NSPoint
    private let nameWidth:CGFloat
    private let amountWidth:CGFloat
    private let tableRightPos0:NSPoint
    private let rowHeight:CGFloat
    private let verticalPadding:CGFloat
    private let textInSet:CGFloat
    
    private let numberOfRows:Int
    
    private let totalPos:NSPoint
    
    var sisanData:[Sisan] = []
    var total:Int
    
    init(sisanData: [Sisan], total:Int){
        
        self.sisanData = sisanData
        
        // A4  20rows per a page   :adjust for A4 size:width 595, height 842
        self.pageHeight = CGFloat(840)
        self.pageWidth = CGFloat(590)
        self.titlePos = NSMakePoint(100, 100)
        self.datePos = NSMakePoint(300, 100)
        self.tableLeftPos0 = NSMakePoint(50, 150)
        self.nameWidth = CGFloat(200)
        self.amountWidth = CGFloat(100.0)
        self.tableRightPos0 = NSMakePoint(50 + self.nameWidth + self.amountWidth , 150)//50+200+100=350
        
        self.rowHeight = CGFloat(25.0)
        self.verticalPadding = CGFloat (4.0)
        self.textInSet = CGFloat(4.0)
        self.numberOfRows = 20 + 1
        
        self.totalPos = NSMakePoint(210, 150 + self.rowHeight * CGFloat(self.numberOfRows + 1))
        
        self.total = total
        
        super.init()
    }
    
    override func draw(with box: PDFDisplayBox) {
        super.draw(with: box)
        self.drawTableLines()
        self.drawTableData()
    }
    
    func drawTableLines(){
        var p00 = tableLeftPos0; var p01 = tableRightPos0
        drawLine(linewidth: CGFloat(1), from: p00, to: p01)
        p00.y += self.rowHeight; p01.y += self.rowHeight
        drawLine(linewidth: CGFloat(1), from: p00, to: p01)
        for _ in 1...3{
            p00.y += self.rowHeight * CGFloat(5); p01.y = p00.y
            drawLine(linewidth: CGFloat(0.3), from: p00, to: p01)
        }
        p00.y += self.rowHeight * CGFloat(5); p01.y = p00.y
        drawLine(linewidth: CGFloat(1), from: p00, to: p01)
        p00 = tableLeftPos0; p01 = NSMakePoint(p00.x, p00.y + rowHeight * CGFloat(numberOfRows))
        drawLine(linewidth: CGFloat(1), from: p00, to: p01)
        p00.x += self.nameWidth ; p01.x = p00.x
        drawLine(linewidth: CGFloat(1), from: p00, to: p01)
        p00.x += self.amountWidth ; p01.x = p00.x
        drawLine(linewidth: CGFloat(1), from: p00, to: p01)
    }
    
    func drawLine(linewidth: CGFloat, from:NSPoint, to:NSPoint){
        let path = NSBezierPath()
        NSColor.darkGray.set()
        //(0,0)left-bottom -> left-top
        let fromC = NSMakePoint(from.x, self.pageHeight - from.y)
        let toC = NSMakePoint(to.x, self.pageHeight - to.y)
        path.move(to: toC)
        path.line(to: fromC)
        path.lineWidth = linewidth
        path.stroke()
    }
    
    func drawTableData(){
        let titleFont = NSFont(name:"Helvetica", size: 14.0)
        var rect0 = makeRect(x: self.titlePos.x, y: self.titlePos.y, w: 120, h: 30)
        "資産レポート".draw(in:rect0, withAttributes:[NSAttributedString.Key.font: titleFont!])
        let dateFont = NSFont(name:"Helvetica", size: 12.0)
        rect0 = makeRect(x: self.datePos.x, y: self.datePos.y, w: 140, h: 20)
        let DF = DateFormatter()
        DF.dateFormat = "YYYY年MM月dd日"
        DF.timeZone = NSTimeZone.local
        let strDate = DF.string(from: Date())
        strDate.draw(in:rect0, withAttributes:[NSAttributedString.Key.font: dateFont!])
        
        let itemFont = NSFont(name:"Helvetica", size: 12.0)
        var p00 = tableLeftPos0
        p00.x  += textInSet; p00.y += (verticalPadding + rowHeight)
        rect0 = makeRect(x: p00.x, y: p00.y, w: 60, h: 20)
        " 名　称".draw(in:rect0, withAttributes:[NSAttributedString.Key.font: itemFont!])
        p00.x += nameWidth
        rect0 = makeRect(x: p00.x, y: p00.y, w: 100, h: 20)
        "   金額（円）".draw(in:rect0, withAttributes:[NSAttributedString.Key.font: itemFont!])
        p00 = self.tableLeftPos0
        p00.x += self.textInSet;p00.y += self.rowHeight * 2
        var p01 = p00; p01.x += (self.nameWidth + self.textInSet * 3)
        var rect1:NSRect
        for line in self.sisanData{
            rect0 = makeRect(x: p00.x, y: p00.y, w: Int(self.nameWidth), h: 20)
            line.Namae.draw(in:rect0, withAttributes:[NSAttributedString.Key.font: itemFont!])
            rect1 = makeRect(x: p01.x, y: p01.y, w: Int(self.amountWidth), h: 20)
            int2stringWithComma(integer: line.Zankin).draw(in:rect1, withAttributes:[NSAttributedString.Key.font: itemFont!])
            p00.y += self.rowHeight; p01.y += self.rowHeight
        }
        
        let strTotal = "合　計　　" + int2stringWithComma(integer: self.total) + " 円"
        p00 = self.totalPos
        rect0 = makeRect(x: p00.x, y: p00.y, w: 160, h: 20)
        strTotal.draw(in:rect0, withAttributes:[NSAttributedString.Key.font: itemFont!])

    }
    func makeRect(x:CGFloat, y:CGFloat, w:Int, h:Int)->NSRect{
        let y0 = self.pageHeight - y
        return NSMakeRect(x, y0, CGFloat(w), CGFloat(h))
    }
    func int2stringWithComma(integer:Int, digits:Int  = 10) -> String{
        var s = ""
        for _ in 1...digits{ s += " " }
        let formatter = NumberFormatter()
        formatter.numberStyle = .decimal
        s += formatter.string(from: integer as NSNumber) ?? "error"
        return String(s.suffix(digits))
    }
}
