//
//  DataModel.swift
//  prjSQLite3
//
//  Created by 梅山 新一郎 on 2018/09/07.
//  Copyright © 2018年 Umeya. All rights reserved.
//

import Cocoa
import SQLite3

class DataModel{
    static let sharedInstance = DataModel()  //...(1)
    var testData1 = [Article]()              //...(2)
    fileprivate var db: OpaquePointer?       //...(3)
    fileprivate var stmt: OpaquePointer?     //...(4)
    fileprivate var strDbPath = Bundle.main.resourcePath! + "/testDb.sql"          //...(5)
    
    private init() {
        openDB()
        guard readDB() else{return}
    }
    
   
    func openDB(){
        if sqlite3_open(strDbPath, &db) != SQLITE_OK{
            let errmsg = String(cString: sqlite3_errmsg(db)!)
            someMessageWithAlert("error open database : \(errmsg)" , "stop the app")
            return
        }
        // create table ARTICLE if not exists
        let sql = "CREATE TABLE IF NOT EXISTS ARTICLE (Id INTEGER NOT NULL PRIMARY KEY,No INTEGER NOT NULL,Name TEXT NOT NULL)"
        if sqlite3_exec(db, sql, nil, nil, nil) != SQLITE_OK{
            let errmsg = String(cString: sqlite3_errmsg(db)!)
            someMessageWithAlert("error when create table, stop the app:\(errmsg)")
        }
    }
    
    func readDB() -> Bool{
        testData1.removeAll()
        
        //this is our seleted query
        let queryString = "SELECT * FROM ARTICLE"
        
        //prepare the query
        if sqlite3_prepare(db, queryString, -1, &stmt, nil) != SQLITE_OK {
            let errmsg = String(cString: sqlite3_errmsg(db)!)
            someMessageWithAlert("error prepare select from ARTICLE: \(errmsg)" , "stop the app")
            return false
        }
        
        //traversing through all records
        while (sqlite3_step(stmt) == SQLITE_ROW) {
            let Id = sqlite3_column_int(stmt, 0)
            let No = sqlite3_column_int(stmt, 1)
            let Name = String(cString: sqlite3_column_text(stmt, 2))
            //adding values to list
            testData1.append(Article(Id:Int(Id), No:Int(No),Name:String(describing: Name)))
        }
        sqlite3_finalize(stmt)
        return true
    }
    
    //binding the parameters
    func bind_text(_ col:Int, _ txt:String) -> Bool {
        let u8txt = (txt as NSString).utf8String
        if sqlite3_bind_text(stmt, Int32(col), u8txt, -1, nil) != SQLITE_OK{
            let errmsg = String(cString: sqlite3_errmsg(db)!)
            someMessageWithAlert("failure binding text: \(errmsg)")
            return false
        }
        return true
    }
    
    func bind_int(_ col:Int, _ i:Int) -> Bool{
        if sqlite3_bind_int(stmt, Int32(col), Int32(i)) != SQLITE_OK{
            let errmsg = String(cString: sqlite3_errmsg(db)!)
            someMessageWithAlert("failure binding int: \(errmsg)")
            return false
        }
        return true
    }
    func insertDB(no:Int, name:String) -> Bool {
        
        //the insert quary
        let queryString = "INSERT INTO ARTICLE (No,Name) VALUES (?,?)"
        
        //preparing the query
        if sqlite3_prepare(db, queryString, -1, &stmt, nil) != SQLITE_OK{
            let errmsg = String(cString: sqlite3_errmsg(db)!)
            someMessageWithAlert("error preparing  insert: \(errmsg)")
            return false
        }
        
        if sqlite3_bind_int(stmt, Int32(1), Int32(no)) != SQLITE_OK{
            let errmsg = String(cString: sqlite3_errmsg(db)!)
            someMessageWithAlert("failure binding int no: \(errmsg)")
            return false
        }
        
        let u8txt = (name as NSString).utf8String
        if sqlite3_bind_text(stmt, Int32(2), u8txt, -1, nil) != SQLITE_OK{
            let errmsg = String(cString: sqlite3_errmsg(db)!)
            someMessageWithAlert("failure binding text name: \(errmsg)")
            return false
        }
        
        //execute the query to insert values
        if sqlite3_step(stmt) != SQLITE_DONE {
            let errmsg = String(cString: sqlite3_errmsg(db)!)
            someMessageWithAlert("failure inserting ARTICLE : \(errmsg)")
            return false
        }
        
        sqlite3_finalize(stmt)
        
        return readDB()
    }
    
    func deleteDB(id:Int) -> Bool{
        let queryString = "DELETE FROM ARTICLE WHERE Id=" + String(id)
        
        //preparing the query
        if sqlite3_prepare(db, queryString, -1, &stmt, nil) != SQLITE_OK{
            let errmsg = String(cString: sqlite3_errmsg(db)!)
            someMessageWithAlert("error preparing delete: \(errmsg)")
            return false
        }
        //execute the query to insert values
        if sqlite3_step(stmt) != SQLITE_DONE {
            let errmsg = String(cString: sqlite3_errmsg(db)!)
            someMessageWithAlert("failure delete from ARTICLE: \(errmsg)")
            return false
        }
        sqlite3_finalize(stmt)
        return readDB()
    }
    
    func updateDB(id:Int, no:Int, name:String)->Bool{
        let queryString = "UPDATE ARTICLE SET no= " + String(no) +  " ,Name= '" + name + "'  WHERE Id=" + String(id)
        if sqlite3_prepare(db, queryString, -1, &stmt, nil) != SQLITE_OK {
            let errmsg = String(cString: sqlite3_errmsg(db)!)
            someMessageWithAlert("error prepare update ARTICLE: \(errmsg)" )
            return false
        }
        //execute the query to insert values
        if sqlite3_step(stmt) != SQLITE_DONE {
            let errmsg = String(cString: sqlite3_errmsg(db)!)
            someMessageWithAlert("failure update: \(errmsg)")
            return false
        }
        sqlite3_finalize(stmt)
        return readDB()
    }
    
    func someMessageWithAlert(_ message: String, _ textInfo:String = "information") {
        let alert = NSAlert()
        alert.messageText = message
        alert.informativeText = textInfo
        alert.runModal()
    }}
