//
//  ViewController.swift
//  prjTableView
//
//  Created by 梅山 新一郎 on 2018/08/30.
//  Copyright © 2018年 Umeya. All rights reserved.
//

import Cocoa

class ViewController: NSViewController, NSTableViewDataSource, NSTableViewDelegate, NSComboBoxDataSource, NSComboBoxDelegate{

    var testData1:[(no: Int, name:String)] = [(10,"梅"), (8,"屋"), (97,"萬"), (2,"年"), (777,"堂")]
    var testData2 = [10:[1,2,3], 8:[11,2], 97:[7,6,9], 2:[20,21,22,23,24], 777:[1000]]
    var vTestData2 = [Int]()
    
    override func viewDidLoad() {
        super.viewDidLoad()

        tblTest1.dataSource = self
        tblTest1.delegate = self
        tblTest2.dataSource = self
        tblTest2.delegate = self
        cmbTest.usesDataSource = true  //...(1)
        cmbTest.dataSource = self         //...(2)
        cmbTest.delegate = self            //...(3)

        
    }

    @IBOutlet weak var cmbTest: NSComboBox!
    @IBOutlet weak var tblTest1: NSTableView!
    @IBOutlet weak var tblTest2: NSTableView!
    
    
    func numberOfItems(in comboBox: NSComboBox) -> Int {
        return vTestData2.count
    }
    
    func comboBox(_ comboBox: NSComboBox, objectValueForItemAt index: Int) -> Any? {
        return vTestData2[index]
    }

    func comboBoxSelectionDidChange(_ notification: Notification) {
        someMessageWithAlert("cmbTestで" + String(vTestData2[cmbTest.indexOfSelectedItem]) + " が選択されました。")
        }
    
    func numberOfRows(in tableView: NSTableView) -> Int {
        if tableView.tag == 0{ // tblTest1
            return testData1.count
        } else {                // tblTest2
            return vTestData2.count
        }
        
    }
    
    func tableView(_ tableView: NSTableView, viewFor tableColumn: NSTableColumn?, row: Int) -> NSView? {
        guard let vw = tableView.makeView(withIdentifier: tableColumn!.identifier, owner: self) as? NSTableCellView else {return nil}

        if tableView.tag == 0 {  //tblTest1
            if tableColumn?.identifier.rawValue == "col1"{
                vw.textField?.integerValue = testData1[row].no
            }else if tableColumn?.identifier.rawValue == "col2"{
                vw.textField?.stringValue = testData1[row].name
            }else{
                return nil
            }
        }else if tableView.tag == 1 {   //tblTest2
            if tableColumn?.identifier.rawValue == "colSuuryou"{
                vw.textField?.integerValue = vTestData2[row]
                
            }else{
                return nil
            }
        }
        return vw
    }
    
    func tableViewSelectionIsChanging(_ notification: Notification) {
        guard let tvw = notification.object as? NSTableView else{return}
        let selRowNo = tvw.selectedRow
        guard selRowNo != -1 else {return}
        
        if tvw.tag == 0{           // tblTest
            txtNoTest1.integerValue = testData1[selRowNo].no  //...(2)
            txtNameTest1.stringValue = testData1[selRowNo].name  //...(3)
            vTestData2 = testData2[testData1[selRowNo].no]!
            tblTest2.reloadData()
        } else if tvw.tag == 1{    // tblTest2
            someMessageWithAlert("tblTest2で、数量" + String(vTestData2[selRowNo]) + "が選択されました。")
        }
    }
    
    
    @IBOutlet weak var txtNoTest1: NSTextField!
    
    @IBOutlet weak var txtNameTest1: NSTextField!
    
    @IBAction func btnAddTest1(_ sender: NSButton) {
        //----------------------------------------------------- data check
        let fldNo = txtNoTest1.stringValue.trimmingCharacters(in: .whitespacesAndNewlines)
        if fldNo.isEmpty || !isStringContainsOnlyNumbers(string: fldNo){
            someMessageWithAlert("番号は数字です")
            return
        }
        let fldName = txtNameTest1.stringValue.trimmingCharacters(in: .whitespacesAndNewlines)
        if fldName.isEmpty{
            someMessageWithAlert("名称が空白です")
            return
        }
        //---------------------------------------------
        testData1.append((Int(fldNo)!, fldName ))  //...(1)
        
        tblTest1.reloadData()                       //...(2)

        
    }
    
    
    @IBAction func btnDeleteTest1(_ sender: NSButton) {
        guard tblTest1.selectedRow != -1 else{          //...(4)
            someMessageWithAlert("データ行を選択してから削除してください。")
            return}
        guard yesNoMessageWithAlert("選択した行のデータを削除しますか？", "削除する", "しない")else{return}         //...(5)
        testData1.remove(at: tblTest1.selectedRow)   //...(6)
        tblTest1.reloadData()            //...(7)
    }
    
    @IBAction func btnEditTest1(_ sender: NSButton) {
        guard tblTest1.selectedRow != -1 else{          //...(8)
            someMessageWithAlert("データ行を選択してから編集更新してください。")
            return}
        guard yesNoMessageWithAlert("選択した行のデータをテキスト・フィールドのデータで更新しますか？", "更新する", "しない")else{return}         //...(9)
        testData1[tblTest1.selectedRow] = (txtNoTest1.integerValue, txtNameTest1.stringValue)   //...(10)
        tblTest1.reloadData()            //...(11)
    }
    
    //-------------for data check
    func someMessageWithAlert(_ message: String, _ textInfo:String = "information") {
        let alert = NSAlert()
        alert.messageText = message
        alert.informativeText = textInfo
        alert.runModal()
    }
    
    func yesNoMessageWithAlert(_ message:String, _ yesTitle:String, _ noTitle:String) -> Bool{
        //Yes: return true, No:return false
        let alert = NSAlert()
        alert.messageText = message
        alert.addButton(withTitle: yesTitle)// = 1000
        alert.addButton(withTitle: noTitle)   // = 1001
        if alert.runModal().rawValue == 1000{
            return true
        }else{
            return false
        }
    }
    
    func isStringContainsOnlyNumbers(string: String) -> Bool {
        return string.trimmingCharacters(in: .decimalDigits).count <= 0
    }
}


//        if tableColumn?.identifier.rawValue == "col1"{
//            vw.textField?.stringValue = String(row) + "行　" + "１列"        }
//        else if tableColumn?.identifier.rawValue == "col2"{
//            vw.textField?.stringValue = String(row) + "行　" + "２列"        }
//        else{
//            return nil
//        }
